/* Parallel Hierarchical Grid -- an adaptive finite element library.
 *
 * Copyright (C) 2005-2010 State Key Laboratory of Scientific and
 * Engineering Computing, Chinese Academy of Sciences. */

/* This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA */

/* $Id: halo.h,v 1.3 2020/12/11 00:34:42 zlb Exp $ */

#ifndef PHG_HALO_H

struct HALO_ {
    ELEMENT	*elist;		/* list of elements */
    INT		*map;		/* neighbours.list[i] => halo->elist[map[i]] */
    COORD	*verts;		/* coordinates of remote vertices */
    INT		nvert_bak;	/* saved number of vertices without halo */
    INT 	nedge_bak;	/* saved number of edges without halo */
    INT 	nface_bak;	/* saved number of faces without halo */
    INT 	nelem_bak;	/* saved number of elems without halo */
    int type;			/* halo type */
};

#define HaloType(size, mask)	(((size) << 3) | (mask))
#define GetHaloMask(type)	((type & (VERT_FLAG | EDGE_FLAG | FACE_FLAG)))
#define GetHaloSize(type)	((type) >> 3)
#define HALO_FACE		HaloType(1, FACE_FLAG)
#define HALO_EDGE		HaloType(1, EDGE_FLAG)
#define HALO_VERT		HaloType(1, VERT_FLAG)

#ifdef __cplusplus
extern "C" {
#endif

ELEMENT *phgGetNeighbour_(GRID *g, ELEMENT *e, int face,
					const char *file, int line);
#define phgGetNeighbour(g, e, face) \
	phgGetNeighbour_(g, e, face, __FILE__, __LINE__)

void phgSetupHalo_(GRID *g, int halo_type, BOOLEAN update_dofs);
#define phgSetupHalo(g, halo_type) phgSetupHalo_(g, halo_type, TRUE)
void phgRemoveHalo(GRID *g);

#ifdef __cplusplus
}
#endif

#define PHG_HALO_H
#endif
